/////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		XSocket.cpp
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			Ingo Griebsch
//
// Version:			1.1
// Erstelldatum:	03.01.2001
// letzte nderung:	22.02.2001
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////
// Copyright 2000 by x-treme
/////////////////////////////////////////////////////////////////////////////////////


// eigene includes
#include "XSocket.h"


bool	XSocket::m_bSocketInitialized = false;
WSADATA XSocket::m_wsaData;


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::XSocket
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Konstruktor
/////////////////////////////////////////////////////////////////////////////////////

XSocket::XSocket()
{
	m_Socket				= 0;

	ZeroMemory(&m_SocketSpecification, sizeof(m_SocketSpecification));
	ZeroMemory(&m_TargetSpecification, sizeof(m_TargetSpecification));
	ZeroMemory(&m_wsaData, sizeof(m_wsaData));
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::XSocket
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Konstruktor
/////////////////////////////////////////////////////////////////////////////////////

XSocket::XSocket(unsigned int nMajor, unsigned int nMinor)
{
	XSocket();

	// und gleich die Windows Socket Version initialisieren
	initialize(nMajor, nMinor);
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::~XSocket
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Destruktor
/////////////////////////////////////////////////////////////////////////////////////

XSocket::~XSocket()
{

}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::initialize
/////////////////////////////////////////////////////////////////////////////////////
// Info:			initialisiert die Windows Socket Version
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::initialize(unsigned int nMajor, unsigned int nMinor)
{
	HRESULT hResult;
	WORD dwVersion;

	if(m_bSocketInitialized)
	{
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::initialize > (ACHTUNG!!!) Windows Socket Dll bereits initialisiert\n");
		#endif
		
		return XSOCKET_OK;
	}
	
	dwVersion = MAKEWORD(nMajor, nMinor);
	
	// WSAStartup erfordert als Gegenstueck WSACleanup - also immmer 
	// uninitialize am Schluss aufrufen
	hResult = WSAStartup(dwVersion, &m_wsaData);

	if (hResult != 0)
	{
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::initialize > (FEHLER!!!) WSAStartup() - Fehler-Nr.: ");
		XLogging::log(hResult);
		XLogging::log("\n");
		#endif

		return hResult;
	}

	// Version und Update
	if (LOBYTE(m_wsaData.wVersion) != nMajor || HIBYTE(m_wsaData.wVersion) != nMinor) 
	{
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::initialize > (FEHLER!!!) Versionnummer - Pruefung fehlgeschlagen\n");
		#endif
		
		// FEHLER - also aufraeumen
		hResult = WSACleanup();
		
		if (hResult == SOCKET_ERROR)
		{
			hResult = WSAGetLastError();
			
			#if defined (XDEBUG) || defined (XSOCKETDEBUG)
			XLogging::log("XSocket::initialize > (FEHLER!!!) WSACleanup() - Fehler-Nr.: ");
			XLogging::log(hResult);
			XLogging::log("\n");
			#endif
			
			return hResult;
		}

		// sicher ist immer besser
		m_bSocketInitialized = false;

		// Fehler zurueck - obwohl WSACleanup erfolgreich
		return WSAVERNOTSUPPORTED;
	}

	// hat geklappt - also Meldung an die Klasse
	m_bSocketInitialized = true;

	#if defined (XDEBUG) || defined (XSOCKETDEBUG)
	XLogging::log("XSocket::initialize > erfolgreich\n");	
	#endif

	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::uninitialize
/////////////////////////////////////////////////////////////////////////////////////
// Info:			de-initialisiert die Windows Socket Version und ent-laedt 
//					somit die lib
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::uninitialize(void)
{
	HRESULT hResult;

	// sind die Windows Sockets ueberhaupt noch initialisiert
	if(!m_bSocketInitialized)
	{
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::uninitialize > (ACHTUNG!!!) Windows Socket Dll nicht initialisiert\n");
		#endif
		
		return XSOCKET_OK;
	}

	// immer sauber bleiben und aufraeumen
	hResult = WSACleanup();

	if (hResult == SOCKET_ERROR)
	{
		hResult = WSAGetLastError();
		
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::uninitialize > (FEHLER!!!) WSACleanup() - Fehler-Nr.: ");
		XLogging::log(hResult);
		XLogging::log("\n");
		#endif
		
		return hResult;
	}

	// auch zum schluss -> immer alles rein halten ;-)
	m_bSocketInitialized = false;
	
	#if defined (XDEBUG) || defined (XSOCKETDEBUG)
	XLogging::log("XSocket::uninitialize > erfolgreich\n");
	#endif

	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::create
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::create(DWORD				dwFlags,
						int					nSocketType, 
						int					nAddressFamily, 
						int					nProtocol, 
						LPWSAPROTOCOL_INFO	lpProtocolInfo)
{
	HRESULT hResult;

	// dann erstellen wir mal den socket
	m_Socket = WSASocket(nAddressFamily, nSocketType, nProtocol, lpProtocolInfo, 0, dwFlags);

	if (m_Socket == INVALID_SOCKET)
	{
		hResult = WSAGetLastError();

		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::create > (FEHLER!!!) WSASocket() - Fehler-Nr.: ");
		XLogging::log(hResult);
		XLogging::log("\n");
		#endif
		
		return hResult;
	}

	// hier noch member der SocketSpecification fuellen ?!
	m_SocketSpecification.sin_family = nAddressFamily;
	
	#if defined (XDEBUG) || defined (XSOCKETDEBUG)
	XLogging::log("XSocket::create > erfolgreich\n");
	#endif
	
	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::close
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Schliesst Socket 
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::close(void)
{
	HRESULT hResult;

	// dann schliessen wir mal wieder das Ding
	hResult = closesocket(m_Socket);

	if(hResult == SOCKET_ERROR)
	{
		hResult = WSAGetLastError();
		
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::close > (FEHLER!!!) closesocket() - Fehler-Nr.: ");
		XLogging::log(hResult);
		XLogging::log("\n");
		#endif

		return hResult;
	}

	#if defined (XDEBUG) || defined (XSOCKETDEBUG)
	XLogging::log("XSocket::close > erfolgreich\n");
	#endif

	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::shutdown
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::shutdown(int nFlags)
{
	HRESULT hResult;

	hResult = ::shutdown(m_Socket, nFlags);
	
	if(hResult == SOCKET_ERROR)
	{
		hResult = WSAGetLastError();
		
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::shutdown > (FEHLER!!!) shutdown() - Fehler-Nr.: ");
		XLogging::log(hResult);
		XLogging::log("\n");
		#endif

		return hResult;
	}

	#if defined (XDEBUG) || defined (XSOCKETDEBUG)
	XLogging::log("XSocket::shutdown > erfolgreich\n");
	#endif

	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::bind
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::bind(LPSTR lpDestName, int nPort)
{
	HRESULT hResult;

	// Spezification unseres Socket bestimmen
	if(lpDestName == NULL)
	{
		m_SocketSpecification.sin_addr.s_addr = htonl(INADDR_ANY);
	}
	else
	{
		// ### hier noch pruefen, ob wandlung korrekt
		m_SocketSpecification.sin_addr.s_addr = inet_addr(lpDestName);
	}

	m_SocketSpecification.sin_port = htons(nPort);

	// binden wir mal - get me baby
	hResult = ::bind(m_Socket, (struct sockaddr*) &m_SocketSpecification, sizeof(m_SocketSpecification));

	if(hResult == SOCKET_ERROR)
	{
		hResult = WSAGetLastError();	

		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::bind > (FEHLER!!!) bind() - Fehler-Nr.: ");
		XLogging::log(hResult);
		XLogging::log("\n");
		#endif

		return hResult;
	}

	#if defined (XDEBUG) || defined (XSOCKETDEBUG)
	XLogging::log("XSocket::bind > erfolgreich\n");
	#endif

	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::connect
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::connect(LPSTR lpDestName, int nPort, int nAddressFamily)
{
	HRESULT hResult;

	// Ziel bestimmen
	m_TargetSpecification.sin_family		= nAddressFamily;
	m_TargetSpecification.sin_port			= htons(nPort);
	m_TargetSpecification.sin_addr.s_addr	= inet_addr(lpDestName);
		
	// und verbinden
	hResult = WSAConnect(m_Socket, 
						 (struct sockaddr*) &m_TargetSpecification, 
						 sizeof(m_TargetSpecification), 
						 NULL, 
						 NULL, 
						 NULL, 
						 NULL);

	if (hResult == SOCKET_ERROR)
	{
		hResult = WSAGetLastError();
		
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::connect > (FEHLER!!!) WSAConnect() - Fehler-Nr.: ");
		XLogging::log(hResult);
		XLogging::log("\n");
		#endif

		return hResult;
	}

	#if defined (XDEBUG) || defined (XSOCKETDEBUG)
	XLogging::log("XSocket::connect > erfolgreich\n");
	#endif
	
	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::listen
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::listen(int nBackLog)
{
	HRESULT hResult;

	hResult = ::listen(m_Socket, nBackLog);

	if(hResult == SOCKET_ERROR)
	{
		hResult = WSAGetLastError();
		
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::listen > (FEHLER!!!) listen() - Fehler-Nr.: ");
		XLogging::log(hResult);
		XLogging::log("\n");
		#endif

		return hResult;
	}

	#if defined (XDEBUG) || defined (XSOCKETDEBUG)
	XLogging::log("XSocket::listen > erfolgreich\n");	
	#endif

	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::accept
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::accept(XSocket &socket)
{
	HRESULT hResult;
	int nTargetSpecificationLength = sizeof(socket.m_TargetSpecification);

	// anfragende Verbindung akzeptieren
	socket.m_Socket = WSAAccept(m_Socket, (struct sockaddr*) &socket.m_TargetSpecification, &nTargetSpecificationLength, NULL, NULL);

	if(socket.m_Socket == INVALID_SOCKET)
	{
		hResult = WSAGetLastError();

		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::accept > (FEHLER!!!) WSAAccept() - Fehler-Nr.: ");
		XLogging::log(hResult);
		XLogging::log("\n");
		#endif

		return hResult;
	}

	#if defined (XDEBUG) || defined (XSOCKETDEBUG)
	XLogging::log("XSocket::accept > erfolgreich\n");
	#endif

	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::select
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::select(fd_set *fdsRead, 
						fd_set *fdsWrite, 
						fd_set *fdsError, 
						struct timeval *tvTimeOut)
{
	HRESULT hResult;

	// struktur vorbereiten
	if(fdsRead != NULL)
	{
		FD_ZERO(fdsRead);
		FD_SET(m_Socket, fdsRead);
	}

	if(fdsWrite != NULL)
	{
		FD_ZERO(fdsWrite);
		FD_SET(m_Socket, fdsWrite);
	}

	if(fdsError != NULL)
	{
		FD_ZERO(fdsError);
		FD_SET(m_Socket, fdsError);
	}

	hResult = ::select(0, fdsRead, fdsWrite, fdsError, tvTimeOut);
	
	// Fehler
	if(hResult == SOCKET_ERROR)
	{
		hResult = WSAGetLastError();

		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::select > (FEHLER!!!) select() - Fehler-Nr.: ");
		XLogging::log(hResult);
		XLogging::log("\n");
		#endif

		return hResult;
	}
	
	// Timeout
	else if(hResult == 0)
	{
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::select > (ACHTUNG!!!) timeout\n");
		#endif
		
		return XSOCKET_TIMEOUT;
	}

	#if defined (XDEBUG) || defined (XSOCKETDEBUG)
	XLogging::log("XSocket::select > erfolgreich\n");
	#endif

	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::receive
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::receive(WSABUF &wsBuffer, DWORD dwBufferCount, DWORD & dwBytesReceived, DWORD &dwFlags)
{
	HRESULT hResult;

	// Daten empfangen und in den bereitgestellten Puffer legen
	hResult = WSARecv(m_Socket, 
						&wsBuffer, 
						dwBufferCount, 
						&dwBytesReceived, 
						&dwFlags, 
						NULL, 
						NULL);

	if(hResult == SOCKET_ERROR)
	{
		hResult = WSAGetLastError();
		
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::receive > (FEHLER!!!) WSARecv() - Fehler-Nr.: ");
		XLogging::log(hResult);
		XLogging::log("\n");
		#endif

		return hResult;
	}

	#if defined (XDEBUG) || defined (XSOCKETDEBUG)
	XLogging::log("XSocket::receive > erfolgreich\n");
	#endif

	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::receiveFrom
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::receiveFrom(LPSTR &lpDestName, 
							 int nDestNameLength, 
							 int &nPort, 
							 WSABUF &wsBuffer, 
							 DWORD dwBufferCount, 
							 DWORD &dwFlags)
{
	HRESULT hResult;
	DWORD	dwBytesReceived;
	int		nTargetLength;
	struct sockaddr_in targetSpecification;
	LPSTR lpDest;
	
	// daten vorbereiten	
	ZeroMemory(&targetSpecification, sizeof(targetSpecification));
	nTargetLength = sizeof(targetSpecification);

	hResult = WSARecvFrom(m_Socket, 
							&wsBuffer, 
							dwBufferCount, 
							&dwBytesReceived, 
							&dwFlags, 
							(struct sockaddr*) &targetSpecification, 
							&nTargetLength, 
							NULL, 
							NULL);

	if(hResult == SOCKET_ERROR)
	{
		hResult = WSAGetLastError();
		
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::receiveFrom > (FEHLER!!!) WSARecvFrom() - Fehler-Nr.: ");
		XLogging::log(hResult);
		XLogging::log("\n");
		#endif

		return hResult;
	}

	// hier die Argumente fuellen - damit man von draussen sieht wer geschickt hat
	lpDest = inet_ntoa(targetSpecification.sin_addr);

	if(lpDest == NULL)
	{
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::receiveFrom > (FEHLER!!!) konnte Adresse nicht konvertieren\n");
		#endif

		return XSOCKET_ERROR;
	}
	
	strncpy(lpDestName, lpDest, nDestNameLength - 1);
	strcat(lpDestName, "\0");
	
	nPort = targetSpecification.sin_port;

	#if defined (XDEBUG) || defined (XSOCKETDEBUG)
	XLogging::log("XSocket::receiveFrom > erfolgreich\n");
	#endif

	return XSOCKET_OK;	
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::send
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::send(WSABUF &wsBuffer, DWORD dwBufferCount, DWORD dwFlags)
{
	HRESULT hResult;
	DWORD	dwBytesSent;

	hResult = WSASend(m_Socket, &wsBuffer, dwBufferCount, &dwBytesSent, dwFlags, NULL, NULL);

	if(hResult == SOCKET_ERROR)
	{
		hResult = WSAGetLastError();
		
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::send > (FEHLER!!!) WSASend() - Fehler-Nr.: ");
		XLogging::log(hResult);
		XLogging::log("\n");
		#endif

		return hResult;
	}

	#if defined (XDEBUG) || defined (XSOCKETDEBUG)
	XLogging::log("XSocket::send > erfolgreich\n");
	#endif

	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::sendTo
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::sendTo(LPSTR lpDestName, int nPort, WSABUF wsBuffer, DWORD dwBufferCount, DWORD dwFlags, int nAddressFamily)
{
	HRESULT hResult;
	DWORD	dwBytesSent;
	struct sockaddr_in targetSpecification;

	// daten vorbereiten
	ZeroMemory(&targetSpecification, sizeof(targetSpecification));

	targetSpecification.sin_family		= nAddressFamily;
	targetSpecification.sin_port		= nPort;
	targetSpecification.sin_addr.s_addr	= inet_addr(lpDestName);

	// und nun ans Ziel senden
	hResult = WSASendTo(m_Socket, 
						&wsBuffer, 
						dwBufferCount, 
						&dwBytesSent, 
						dwFlags, 
						(struct sockaddr*) &targetSpecification, 
						sizeof(targetSpecification), 
						NULL, 
						NULL);

	if(hResult == SOCKET_ERROR)
	{
		hResult = WSAGetLastError();
		
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::sendTo > (FEHLER!!!) WSASendTo() - Fehler-Nr.: ");
		XLogging::log(hResult);
		XLogging::log("\n");
		#endif

		return hResult;
	}

	#if defined (XDEBUG) || defined (XSOCKETDEBUG)
	XLogging::log("XSocket::sendTo > erfolgreich\n");
	#endif

	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::setOptions
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::setOptions(int nOptionLevel, int nOptionName, char *cOptionValue, int nOptionValueLength)
{
	HRESULT hResult;

	hResult = setsockopt(m_Socket, nOptionLevel, nOptionName, cOptionValue, nOptionValueLength);

	if(hResult == SOCKET_ERROR)
	{
		hResult = WSAGetLastError();
		
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::setOptions > (FEHLER!!!) setsockopt() - Fehler-Nr.: ");
		XLogging::log(hResult);
		XLogging::log("\n");
		#endif

		return hResult;
	}

	#if defined (XDEBUG) || defined (XSOCKETDEBUG)
	XLogging::log("XSocket::setOptions > erfolgreich\n");
	#endif
	
	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::getOptions
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::getOptions(int nOptionLevel, 
							int nOptionName, 
							char *cOptionValue, 
							int nOptionValueLength)
{
	HRESULT hResult;

	hResult = getsockopt(m_Socket, 
							nOptionLevel, 
							nOptionName, 
							cOptionValue, 
							&nOptionValueLength);

	if(hResult == SOCKET_ERROR)
	{
		hResult = WSAGetLastError();
		
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::getOptions > (FEHLER!!!) getsockopt() - Fehler-Nr.: ");
		XLogging::log(hResult);
		XLogging::log("\n");
		#endif

		return hResult;
	}

	#if defined (XDEBUG) || defined (XSOCKETDEBUG)
	XLogging::log("XSocket::getOptions > erfolgreich\n");
	#endif

	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::isReadable
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::isReadable(bool &bReadable, struct timeval *tvTimeOut)
{
	HRESULT hResult;

	fd_set fdSet;
	
	FD_ZERO(&fdSet);
	FD_SET(m_Socket, &fdSet);

	hResult = ::select(0, &fdSet, NULL, NULL, tvTimeOut);

	// Fehler
	if(hResult == SOCKET_ERROR)
	{
		// konnte status nicht ermitteln
		bReadable = false;

		hResult = WSAGetLastError();

		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::isReadable > (FEHLER!!!) select() - Fehler-Nr.: ");
		XLogging::log(hResult);
		XLogging::log("\n");
		#endif

		return hResult;
	}
	
	// Timeout
	else if(hResult == 0)
	{
		// konnte status nicht ermitteln
		bReadable = false;

		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::isReadable > (ACHTUNG!!!) timeout\n");
		#endif
		
		return XSOCKET_TIMEOUT;
	}

	// hat geklappt - der socket ist in lesbarem zustand
	bReadable = true;

	#if defined (XDEBUG) || defined (XSOCKETDEBUG)
	XLogging::log("XSocket::isReadable > erfolgreich\n");
	#endif

	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::isWriteable
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::isWriteable(bool &bWriteable, struct timeval *tvTimeOut)
{
	HRESULT hResult;

	fd_set fdSet;
	
	FD_ZERO(&fdSet);
	FD_SET(m_Socket, &fdSet);

	hResult = ::select(0, NULL, &fdSet, NULL, tvTimeOut);

	// Fehler
	if(hResult == SOCKET_ERROR)
	{
		// konnte status nicht ermitteln		
		bWriteable = false;

		hResult = WSAGetLastError();

		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::isWriteable > (FEHLER!!!) select() - Fehler-Nr.: ");
		XLogging::log(hResult);
		XLogging::log("\n");
		#endif

		return hResult;
	}
	
	// Timeout
	else if(hResult == 0)
	{
		// konnte status nicht ermitteln
		bWriteable = false;

		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::isWriteable > (ACHTUNG!!!) timeout\n");
		#endif
		
		return XSOCKET_TIMEOUT;
	}

	// hat geklappt - der socket ist in schreibbarem zustand
	bWriteable = true;

	#if defined (XDEBUG) || defined (XSOCKETDEBUG)
	XLogging::log("XSocket::isWriteable > erfolgreich\n");
	#endif

	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::getHostByAddress
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::getBufferSize(unsigned long &ulBufferSize)
{
	HRESULT hResult;

	hResult = ioctlsocket(m_Socket, FIONREAD, &ulBufferSize);

	if (hResult == SOCKET_ERROR)
	{
		hResult = WSAGetLastError();

		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::getBufferSize > (FEHLER!!!) ioctlsocket() - Fehler-Nr.: ");
		XLogging::log(hResult);
		XLogging::log("\n");
		#endif

		return hResult;
	}

	#if defined (XDEBUG) || defined (XSOCKETDEBUG)
	XLogging::log("XSocket::getBufferSize > erfolgreich\n");
	#endif
	
	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::getHostByAddress
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::getHostByAddress(HOSTENT *&hostent, 
								  LPSTR lpAddress, 
								  int nAddressLength, 
								  int nAddressType)
{
	HRESULT hResult;
	unsigned long ulAddress;

	// zuerst String wandeln
	ulAddress = inet_addr(lpAddress);
	
	// keine gueltige addresse ?
	if(ulAddress == INADDR_NONE)
	{
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::getHostByAddress > (FEHLER!!!) keine gueltige Adresse\n");
		#endif

		return XSOCKET_ERROR;
	}

	// informationen holen
	hostent = gethostbyaddr((LPSTR) &ulAddress, nAddressLength, nAddressType);

	// keine infos bekommen ?
	if(hostent == NULL)
	{
		hResult = WSAGetLastError();
		
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::getHostByAddress > (FEHLER!!!) gethostbyaddr() - Fehler-Nr.: ");
		XLogging::log(hResult);
		XLogging::log("\n");
		#endif

		return hResult;
	}

	#if defined (XDEBUG) || defined (XSOCKETDEBUG)
	XLogging::log("XSocket::getHostByAddress > erfolgreich\n");
	#endif

	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::getHostByAddress
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::getHostByAddress(LPSTR lpHostName, 
								  int nHostNameLength,
								  LPSTR lpAddress, 
								  int nAddressLength, 
								  int nAddressType)
{
	HRESULT			hResult;
	HOSTENT			*hostent;
	unsigned long	ulAddress;

	// zuerst String wandeln
	ulAddress = inet_addr(lpAddress);
	
	// keine gueltige addresse ?
	if(ulAddress == INADDR_NONE)
	{
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::getHostByAddress > (FEHLER!!!) keine gueltige Adresse\n");
		#endif

		lpHostName = NULL;

		return XSOCKET_ERROR;
	}

	// informationen holen
	hostent = gethostbyaddr((LPSTR) &ulAddress, nAddressLength, nAddressType);

	// keine infos bekommen ?
	if(hostent == NULL)
	{
		hResult = WSAGetLastError();
		
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::getHostByAddress > (FEHLER!!!) gethostbyaddr() - Fehler-Nr.: ");
		XLogging::log(hResult);
		XLogging::log("\n");
		#endif

		lpHostName = NULL;

		return hResult;
	}

	// Addresse aus den Infos filtern
	strncpy(lpHostName, hostent->h_name, nHostNameLength - 1);
	strcat(lpHostName, "\0");

	#if defined (XDEBUG) || defined (XSOCKETDEBUG)
	XLogging::log("XSocket::getHostByAddress > erfolgreich\n");
	#endif

	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::getHostByName
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::getHostByName(HOSTENT *&hostent, LPSTR lpHostName)
{
	HRESULT hResult;

	hostent = gethostbyname(lpHostName);

	if(hostent == NULL)
	{
		hResult = WSAGetLastError();
		
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::getHostByName > (FEHLER!!!) gethostbyname() - Fehler-Nr.: ");
		XLogging::log(hResult);
		XLogging::log("\n");
		#endif

		return hResult;
	}

	#if defined (XDEBUG) || defined (XSOCKETDEBUG)
	XLogging::log("XSocket::getHostByName > erfolgreich\n");
	#endif

	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::getHostByName
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::getHostByName(LPSTR lpAddress, int nAddressLength, LPSTR lpHostName)
{
	HRESULT hResult;
	HOSTENT *hostent;
	LPSTR	lpDummy;

	hostent = gethostbyname(lpHostName);

	if(hostent == NULL)
	{
		hResult = WSAGetLastError();
		
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::getHostByName > (FEHLER!!!) gethostbyname() - Fehler-Nr.: ");
		XLogging::log(hResult);
		XLogging::log("\n");
		#endif
		
		lpAddress = NULL;

		return hResult;
	}

	/*	Adresse aus den Infos filtern
		von hinten nach vorne...
		hostent->h_addr_list ist ein pointer auf einen pointer auf einen char
		wird de - referenziert
		wird anschliessend in PIN_ADDR gecastet
		wird anschliessend de - referenziert
	*/
	
	lpDummy = inet_ntoa( *(PIN_ADDR) *(hostent->h_addr_list) );

	// hat nicht geklappt ?
	if(lpDummy == NULL)
	{
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::getHostByName > (FEHLER!!!) konnte Name nicht ermitteln\n");
		#endif
	
		lpAddress = NULL;

		return XSOCKET_ERROR;
	}
	
	// Infos bereitstellen
	strncpy(lpAddress, lpDummy, nAddressLength - 1);
	strcat(lpAddress, "\0");

	#if defined (XDEBUG) || defined (XSOCKETDEBUG)
	XLogging::log("XSocket::getHostByName > erfolgreich\n");
	#endif

	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::getLocalName
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::getLocalName(LPSTR lpLocalName, int nLocalNameLength)
{
	HRESULT hResult;

	hResult = gethostname(lpLocalName, nLocalNameLength);

	if(hResult == SOCKET_ERROR)
	{
		hResult = WSAGetLastError();
		
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::getHostName > (FEHLER!!!) gethostname() - Fehler-Nr.: ");
		XLogging::log(hResult);
		XLogging::log("\n");
		#endif

		return hResult;
	}
	
	#if defined (XDEBUG) || defined (XSOCKETDEBUG)
	XLogging::log("XSocket::getHostName > erfolgreich\n");
	#endif
	
	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::getLocalAddress
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::getLocalAddress(LPSTR lpLocalAddress, int nLocalAddressLength)
{
	HRESULT hResult;
	char cHost[255];
	int nHostLength = sizeof(cHost);

	hResult = gethostname(cHost, nHostLength);

	if(hResult == SOCKET_ERROR)
	{
		hResult = WSAGetLastError();
		
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::getHostAddress > (FEHLER!!!) gethostname() - Fehler-Nr.: ");
		XLogging::log(hResult);
		XLogging::log("\n");
		#endif

		return hResult;
	}

	hResult = getHostByName(lpLocalAddress, nLocalAddressLength, cHost);

	if(hResult != XSOCKET_OK)
	{
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::getHostAddress > (FEHLER!!!) getHostByName() - Fehler\n");
		#endif

		return hResult;
	}

	#if defined (XDEBUG) || defined (XSOCKETDEBUG)
	XLogging::log("XSocket::getHostAddress > erfolgreich\n");
	#endif

	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::getLocalPort
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::getLocalPort
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::getLocalPort(LPSTR lpLocalPort, int nLocalPortLength)
{
	int nPort = ntohs(m_SocketSpecification.sin_port);
	char* cPort = new char[16];

	_itoa(nPort, cPort, 10);

	strncpy(lpLocalPort, cPort, nLocalPortLength - 1);
	strcat(lpLocalPort, "\0");
	
	delete cPort;
	
	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::getPeerName
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::getPeerName(LPSTR lpPeerName, int nPeerNameLength)
{
	HRESULT hResult;
	LPSTR lpPeerAddress;

	struct sockaddr_in peer;
	int nPeerLength;
	
	// struktur vorbereiten
	ZeroMemory(&peer, nPeerLength = sizeof(peer));

	// Addresse des Gegenstuecks ermitteln
	hResult = getpeername(m_Socket, (struct sockaddr*) &peer, &nPeerLength);

	if(hResult == SOCKET_ERROR)
	{
		hResult = WSAGetLastError();
		
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::getPeerName > (FEHLER!!!) getpeername() - Fehler-Nr.: ");
		XLogging::log(hResult);
		XLogging::log("\n");
		#endif

		lpPeerName = NULL;

		return hResult;
	}

	// Adresse in String wandeln
	lpPeerAddress = inet_ntoa(peer.sin_addr); // lpPeerAddress

	if(lpPeerAddress == NULL) // lpPeerAddress
	{
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::getPeerName > (FEHLER!!!) konnte Adresse nicht konvertieren\n");
		#endif

		return XSOCKET_ERROR;
	}

	// Namen anahnd Adresse ermitteln 
	hResult = getHostByAddress(lpPeerName, nPeerNameLength, lpPeerAddress);

	if(hResult != XSOCKET_OK)
	{
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::getPeerName > (FEHLER!!!) getHostByAddress() - Fehler\n");
		#endif

		return hResult;
	}

	#if defined (XDEBUG) || defined (XSOCKETDEBUG)
	XLogging::log("XSocket::getPeerName > erfolgreich\n");
	#endif

	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::getpeerAddress
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::getPeerAddress(LPSTR lpPeerAddress, int nPeerAddressLength)
{
	HRESULT hResult;
	LPSTR lpAddress;

	struct sockaddr_in peer;
	int nPeerLength;
	
	// struktur vorbereiten
	ZeroMemory(&peer, nPeerLength = sizeof(peer));

	// Addresse des Gegenstuecks ermitteln
	hResult = getpeername(m_Socket, (struct sockaddr*) &peer, &nPeerLength);

	if(hResult == SOCKET_ERROR)
	{
		hResult = WSAGetLastError();
		
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::getPeerAddress > (FEHLER!!!) getpeername() - Fehler-Nr.: ");
		XLogging::log(hResult);
		XLogging::log("\n");
		#endif

		lpPeerAddress = NULL;

		return hResult;
	}

	// Adresse in String wandeln
	lpAddress = inet_ntoa(peer.sin_addr);

	if(lpAddress == NULL)
	{
		#if defined (XDEBUG) || defined (XSOCKETDEBUG)
		XLogging::log("XSocket::getPeerAddress> (FEHLER!!!) konnte Adresse nicht konvertieren\n");
		#endif

		return XSOCKET_ERROR;
	}

	strncpy(lpPeerAddress, lpAddress, nPeerAddressLength - 1);
	strcat(lpPeerAddress, "\0");

	#if defined (XDEBUG) || defined (XSOCKETDEBUG)
	XLogging::log("XSocket::getPeerAddress > erfolgreich\n");
	#endif

	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSocket::getpeerAddress
/////////////////////////////////////////////////////////////////////////////////////
// Info:			
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSocket::getPeerPort(LPSTR lpPeerPort, int nPeerPortLength)
{
	int nPort = ntohs(m_TargetSpecification.sin_port);
	char* cPort = new char[16];

	_itoa(nPort, cPort, 10);

	strncpy(lpPeerPort, cPort, nPeerPortLength - 1);
	strcat(lpPeerPort, "\0");
	
	delete cPort;

	return XSOCKET_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	XSocket.cpp
/////////////////////////////////////////////////////////////////////////////////////
