////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		XSocket.h
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			Ingo Griebsch
//
// Version:			1.1
// Erstelldatum:	03.01.2001
// letzte nderung:	22.02.2001
/////////////////////////////////////////////////////////////////////////////////////
// Info:
/////////////////////////////////////////////////////////////////////////////////////
// Copyright 2000 by x-treme
/////////////////////////////////////////////////////////////////////////////////////


#define WIN32_LEAN_AND_MEAN		// Selten genutzte Teile aus Win-Header ausschliessen

#ifndef _XSOCKET_H
#define _XSOCKET_H


// eigene defines
#define XSOCKET			WM_USER + 1
#define XSOCKET_OK		XSOCKET + 1
#define XSOCKET_ERROR	XSOCKET + 2
#define XSOCKET_TIMEOUT XSOCKET + 3


// standard includes
#include <winsock2.h>


// XGame includes
#include "XLogging.h"


// Klassen-Deklaration

class XSocket 
{
	public:
		// Konstruktor		
		XSocket();

		// Konstruktor der initialize aufruft
		XSocket(unsigned int nMajor, unsigned int nMinor);
		
		// Destruktor
		virtual ~XSocket();

		// initialize - initialisiert Windows Socket Version
		static HRESULT initialize(unsigned int nMajor = 1, unsigned int nMinor = 1);

		// uninitialize - deinitialisiert Windows Socket Version
		static HRESULT uninitialize(void);

		// erstellt Socket zu anzugebenden Service Provider
		HRESULT create(DWORD dwFlags						= WSA_FLAG_OVERLAPPED,
						int nSocketType						= SOCK_STREAM, 
						int nAddressFamily					= AF_INET, 
						int nProtocol						= 0, 
						LPWSAPROTOCOL_INFO lpProtocolInfo	= NULL);

		// schliesst Socket
		HRESULT close(void);

		// sperrt Socket fr Lesen und/oder Schreiben
		HRESULT shutdown(int nFlags = SD_BOTH);

		// assoziiert eine lokale Adresse mit dem Socket
		HRESULT bind(LPSTR lpDestName = NULL, int nPort = 1024);

		// stellt Verbindung her
		HRESULT connect(LPSTR lpDestName, int nPort = 1024, int nAddressFamily = AF_INET);

		// versetzt Socket in Horch-Status
		HRESULT listen(int nBacklog = SOMAXCONN);

		// akzeptiert eingehende Verbindung
		HRESULT accept(XSocket &socket);

		// prueft den Status des Socket auf Lesbarkeit und/oder Schreibbarkeit und/oder Fehler
		HRESULT select(fd_set *fdsRead, 
						fd_set *fdsWrite			= NULL, 
						fd_set *fdsError			= NULL, 
						struct timeval *tvTimeOut	= NULL);

		// empfaengt Daten des verbundenen Sockets
		HRESULT receive(WSABUF &wsBuffer, DWORD dwBufferCount, DWORD &dwBytesReceived, DWORD &dwFlags);

		// empfaengt Daten und gibt Sender bekannt
		HRESULT receiveFrom(LPSTR &lpDestName,
							int nDestNameLength,
							int &nPort, 
							WSABUF &wsBuffer, 
							DWORD dwBufferCount, 
							DWORD &dwFlags);

		// sendet Daten zu verbundenem Socket
		HRESULT send(WSABUF &wsBuffer, DWORD dwBufferCount, DWORD dwFlags);

		// sendet Daten an anzugebendes Ziel
		HRESULT sendTo(LPSTR lpDestName,
						int nPort, 
						WSABUF wsBuffer, 
						DWORD dwBufferCount, 
						DWORD dwFlags, 
						int nAddressFamily = AF_INET);

		// setzt zu uebergebende Optionen fuer den Socket
		HRESULT setOptions(int nOptionLevel, 
							int nOptionName, 
							char *cOptionValue, 
							int nOptionValueLength);

		// macht Optionen fuer den Socket bekannt
		HRESULT getOptions(int nOptionLevel, 
							int nOptionName, 
							char *cOptionValue, 
							int nOptionValueLength);
		
		// prueft, ob Socket in lesbarem Zustand ist 
		// kann mittels bReadable geprueft werden
		HRESULT isReadable(bool &bReadable, struct timeval *tvTimeOut = NULL);

		// prueft, ob Socket in schreibbarem Zustand ist
		// kann mittels bWriteable geprueft werden
		HRESULT isWriteable(bool &bWriteable, struct timeval *tvTimeOut = NULL);
		
		// ermittelt die Grsse des Puffers, der an receive uebergeben werden muss
		HRESULT getBufferSize(unsigned long &ulBufferSize);
		
		// fuellt String mit Namen des Rechners der den Socket aufsetzt
		static HRESULT getLocalName(LPSTR lpLocalName, int nHostNameLength);

		// fuellt String mit Ip-Adresse des Rechners der den Socket aufsetzt
		static HRESULT getLocalAddress(LPSTR lpLocalAddress, int nHostAddressLength);

		// fuellt String mit Port des Rechners der den Socket aufsetzt
		HRESULT getLocalPort(LPSTR lpLocalPort, int nLocalPortLength);

		// fuellt String mit Namen des Rechners der den verbundenen Socket aufsetzt
		HRESULT getPeerName(LPSTR lpPeerName, int nPeerNameLength);

		// fuellt String mit Ip-Addresse des Rechners der den verbundenen Socket aufsetzt
		HRESULT getPeerAddress(LPSTR lpPeerAddress, int nPeerAddressLength);

		// fuellt String mit Port des Rechners der den verbundenen Socket aufsetzt
		HRESULT getPeerPort(LPSTR lpPeerPort, int nPeerPortLength);

		
		/*	
			hier folgen sogenannte DB-Funktionen, die von der Windows Socket API zur
			Verfuegung gestellt werden und sind als statische Methoden realisiert.
		*/

		// fuellt HOSTENT-Strukur mit Informationen des Rechners mit der zu uebergebenden Adresse
		static HRESULT getHostByAddress(HOSTENT *&hostent, 
										LPSTR lpAddress, 
										int nAddressLength = 4, 
										int nAddressType = AF_INET);

		// fuellt String mit Namen des Rechners mit der zu uebergebenden Adresse
		static HRESULT getHostByAddress(LPSTR lpHostName,
										int nHostNameLength,
										LPSTR lpAddress, 
										int nAddressLength = 4, 
										int nAddressType = AF_INET);
		
		// fuellt HOSTENT-Strukur mit Informationen des Rechners mit dem zu uebergebenden Namen
		static HRESULT getHostByName(HOSTENT *&hostent, LPSTR lpHostName);

		// fuellt String mit Ip-Addresse des Rechners mit dem zu uebergebenden Namen
		static HRESULT getHostByName(LPSTR lpAddress, int nAddressLength, LPSTR lpHostName);

	protected:
		// Socket - Handle 
		SOCKET		m_Socket;

		// spezifiziert diesen Socket
		sockaddr_in	m_SocketSpecification;

		// spezifiziert Ziel
		sockaddr_in m_TargetSpecification;

		// spezifiziert Windows Socket Version
		static WSAData	m_wsaData;

		// boolsche Member um Interna zu pruefen
		static bool	m_bSocketInitialized;

	private:

};


#endif


class XSocket;


/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	XSocket.h
/////////////////////////////////////////////////////////////////////////////////////


