/////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		XSprite.cpp
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			pHreak
//
// Erstelldatum:	04.09.2000
// letzte Aenderung:15.03.2001
/////////////////////////////////////////////////////////////////////////////////////
// Info:			XSprite verwaltet ein Bitmap-Sprite, ermoeglicht dessen
//					Darstellung in einer XSurface.
/////////////////////////////////////////////////////////////////////////////////////
// Copyright 2000/2001 by x-treme
/////////////////////////////////////////////////////////////////////////////////////

#define DIRECTDRAW_VERSION 0x0700 // Dann klappts auch mit dem "DirectX>7-SDK"-Nachbarn

// XGame-includes
#include "XDisplay.h"
#include "XSprite.h"

// Methoden-Definitionen

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSprite::XSprite
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Konstruktor von XSprite, initialisiert alle Datenmember der
//					Klasse.
/////////////////////////////////////////////////////////////////////////////////////

XSprite::XSprite(void)
{
	resetData();
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSprite::resetData
/////////////////////////////////////////////////////////////////////////////////////
// Info:			setzt alle Datenmember der Klasse zurueck
/////////////////////////////////////////////////////////////////////////////////////

void XSprite::resetData(void)
{
	m_bIsInitialized	= false;
	m_bVisible			= false;
	m_lFrame			= 0;
	m_lFrameHeight		= 0;
	m_lFramesPerRow		= 0;
	m_lFrameWidth		= 0;
	m_lNrOfFrames		= 0;
	m_lpstrFilename		= NULL;
	m_lX				= 0;
	m_lY				= 0;
	m_lZ				= 0;
	m_pXSurfaceSprite	= NULL;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSprite::~XSprite
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Destruktor von XSprite, ruft XSprite::uninitialize auf
/////////////////////////////////////////////////////////////////////////////////////

XSprite::~XSprite(void)
{
	// Die Pruefung, ob das Objekt initialisiert ist, findet schon hier statt damit
	// keine WARNUNG ausgegeben wird, wenn ein Objekt erzeugt, aber nie verwendet
	// wird. Siehe hierzu auch XSprite::uninitialize.
	
	if (m_bIsInitialized==true)
	{
		uninitialize();
	}
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSprite::initialize
/////////////////////////////////////////////////////////////////////////////////////
// Info:			erzeugt eine neues Sprite gemaess den Parametern der Datei
//					lpstrSpritefile
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSprite::initialize(XDisplay*	pXDisplay,
							LPSTR		lpstrSpritefile)
{
	HRESULT		hDDResult;
	const		STRLEN = 1024;	
	char		cReturned[STRLEN];
	HFILE		hFile;
	OFSTRUCT	ReOpenBuff;
	DDCOLORKEY	DDColorKey;
	char		cSeparators[]	= ", ";
	char*		pcToken;
	long		lCount;
	WORD		wRGB[3]			= {0,0,0};

	#if defined(XDEBUG) || defined(XSPRITEDEBUG)
	if (m_bIsInitialized==true)
	{
		XLogging::log("XSprite::initialize > (FEHLER!!!) Sprite bereits initialisiert\n");
		
		return DDERR_INVALIDOBJECT;
	}
	if ((!pXDisplay) || (!pXDisplay->m_lpDirectDraw7))
	{
		XLogging::log("XSprite::initialize > (FEHLER!!!) DirectDraw7-Objekt existiert nicht\n");
		
		return DDERR_INVALIDOBJECT;
	}
	#endif

	// Pruefen, ob Datei vorhanden
	hFile = OpenFile(lpstrSpritefile,&ReOpenBuff,OF_EXIST);
	
	if (hFile==HFILE_ERROR)
	{
		#if defined(XDEBUG) || defined(XSPRITEDEBUG)
		XLogging::log("XSprite::initialize > (FEHLER!!!) Spritedatei \"");
		XLogging::log(lpstrSpritefile);
		XLogging::log("\" nicht gefunden\n");
		#endif

		return DDERR_NOTFOUND;
	}
	
	// ColorKey holen
	GetPrivateProfileString("XSprite","ColorKey","",cReturned,STRLEN,lpstrSpritefile);

	if (!strlen(cReturned))
	{
		#if defined(XDEBUG) || defined(XSPRITEDEBUG)
		XLogging::log("XSprite::initialize > (FEHLER!!!) Schlssel ColorKey in Datei \"");
		XLogging::log(lpstrSpritefile);
		XLogging::log("\" existiert nicht\n");
		#endif

		_lclose(hFile);

		return DDERR_NOTFOUND;
	}

	lCount	= 0;
	pcToken = strtok(cReturned,cSeparators);
	
	while ((pcToken!=NULL) && (lCount<3))
	{
		wRGB[lCount]	= (WORD) atol(pcToken);	
		lCount++;
		pcToken	= strtok(NULL,cSeparators);
	}

	if (lCount!=3)
	{
		#if defined(XDEBUG) || defined(XSPRITEDEBUG)
		XLogging::log("XSprite::initialize > (WARNUNG!!!) ColorKey-Daten ungltig \"");
		XLogging::log(lpstrSpritefile);
		XLogging::log("\"\n");
		#endif
	}

	DDColorKey.dwColorSpaceHighValue	=
	DDColorKey.dwColorSpaceLowValue		= XDisplay::convertRGBToColorModel(wRGB[0],wRGB[1],wRGB[2]);

	// SpriteBitmap holen
	GetPrivateProfileString("XSprite","SpriteBitmap","",cReturned,STRLEN,lpstrSpritefile);

	if (!strlen(cReturned))
	{
		#if defined(XDEBUG) || defined(XSPRITEDEBUG)
		XLogging::log("XSprite::initialize > (FEHLER!!!) Schlssel SpriteBitmap in Datei \"");
		XLogging::log(lpstrSpritefile);
		XLogging::log("\" existiert nicht\n");
		#endif

		_lclose(hFile);

		return DDERR_NOTFOUND;
	}
	
	// Surface allozieren
	m_pXSurfaceSprite = new XSurface;
	if (!m_pXSurfaceSprite)
	{
		#if defined(XDEBUG) || defined(XSPRITEDEBUG)
		XLogging::log("XSprite::initialize > (FEHLER!!!) Zuwenig Speicher fr Surface \"");
		XLogging::log(lpstrSpritefile);
		XLogging::log("\"\n");
		#endif

		_lclose(hFile);

		return DDERR_OUTOFMEMORY;
	}
	
	// Surface mit Spritedaten fuellen
	hDDResult = m_pXSurfaceSprite->initialize(pXDisplay,cReturned,&DDColorKey);

	if (hDDResult!=DD_OK)
	{
		_lclose(hFile);

		return hDDResult;
	}

	// Initialisierung erfolgt
	m_bIsInitialized	= true;

	// Dateinamen kopieren
	m_lpstrFilename = new char[strlen(lpstrSpritefile)+1];
	if (!m_lpstrFilename)
	{
		#if defined(XDEBUG) || defined(XSPRITEDEBUG)
		XLogging::log("XSprite::initialize > (FEHLER!!!) Zuwenig Speicher fr Filename\n");
		#endif

		_lclose(hFile);

		uninitialize();

		return DDERR_OUTOFMEMORY;
	}
	strcpy(m_lpstrFilename,lpstrSpritefile);

	// Anzahl Frames holen
	GetPrivateProfileString("XSprite","Frames","",cReturned,STRLEN,lpstrSpritefile);

	if (!strlen(cReturned))
	{
		#if defined(XDEBUG) || defined(XSPRITEDEBUG)
		XLogging::log("XSprite::initialize > (FEHLER!!!) Schlssel Frames in Datei \"");
		XLogging::log(lpstrSpritefile);
		XLogging::log("\" existiert nicht\n");
		#endif

		_lclose(hFile);

		uninitialize();

		return DDERR_NOTFOUND;
	}

	m_lNrOfFrames = atol(cReturned);
	
	// Breite eines Frames holen
	GetPrivateProfileString("XSprite","FrameWidth","",cReturned,STRLEN,lpstrSpritefile);

	if (!strlen(cReturned))
	{
		#if defined(XDEBUG) || defined(XSPRITEDEBUG)
		XLogging::log("XSprite::initialize > (FEHLER!!!) Schlssel FrameWidth in Datei \"");
		XLogging::log(lpstrSpritefile);
		XLogging::log("\" existiert nicht\n");
		#endif

		_lclose(hFile);

		uninitialize();

		return DDERR_NOTFOUND;
	}

	m_lFrameWidth = atol(cReturned);
	
	// Hoehe eines Frames holen
	GetPrivateProfileString("XSprite","FrameHeight","",cReturned,STRLEN,lpstrSpritefile);

	if (!strlen(cReturned))
	{
		#if defined(XDEBUG) || defined(XSPRITEDEBUG)
		XLogging::log("XSprite::initialize > (FEHLER!!!) Schlssel FrameHeight in Datei \"");
		XLogging::log(lpstrSpritefile);
		XLogging::log("\" existiert nicht\n");
		#endif

		_lclose(hFile);

		uninitialize();

		return DDERR_NOTFOUND;
	}

	m_lFrameHeight = atol(cReturned);
	
	m_lFramesPerRow = m_pXSurfaceSprite->getWidth()/m_lFrameWidth;
	
	#if defined(XDEBUG) || defined(XSPRITEDEBUG)
	if (m_lFramesPerRow!=m_pXSurfaceSprite->getWidth()/(float) m_lFrameWidth)
	{
		XLogging::log("XSprite::initialize > (WARNUNG!!!) Bitmap- und Framebreite passen nicht zueinander \"");
		XLogging::log(lpstrSpritefile);
		XLogging::log("\"\n");
	}
	
	XLogging::log("XSprite::initialize > Sprite erzeugt \"");
	XLogging::log(m_lpstrFilename);
	XLogging::log("\"\n");
	#endif

	_lclose(hFile);

	return hDDResult;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSprite::initialize
/////////////////////////////////////////////////////////////////////////////////////
// Info:			erzeugt eine neues Sprite, das optisch Sprite pxSprite entspricht
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSprite::initialize(XSprite*	pXSprite)
{
	if (m_bIsInitialized==true)
	{
		#if defined(XDEBUG) || defined(XSURFACEDEBUG)
		XLogging::log("XSprite::initialize > (FEHLER!!!) Sprite bereits initialisiert\n");
		#endif
		
		return DDERR_INVALIDOBJECT;
	}

	if ((!pXSprite) || (pXSprite->m_bIsInitialized==false))
	{
		#if defined(XDEBUG) || defined(XSPRITEDEBUG)
		XLogging::log("XSprite::initialize > (FEHLER!!!) Spritevorlage ungltig\n");
		#endif
		
		return DDERR_INVALIDOBJECT;
	}

	m_lpstrFilename = new char[strlen(pXSprite->m_lpstrFilename)+1];
	if (!m_lpstrFilename)
	{
		#if defined(XDEBUG) || defined(XSPRITEDEBUG)
		XLogging::log("XSprite::initialize > (FEHLER!!!) Zuwenig Speicher fr Filename\n");
		#endif

		return DDERR_OUTOFMEMORY;
	}
	strcpy(m_lpstrFilename,pXSprite->m_lpstrFilename);

	m_lFrameHeight		= pXSprite->m_lFrameHeight;
	m_lFramesPerRow		= pXSprite->m_lFramesPerRow;
	m_lFrameWidth		= pXSprite->m_lFrameWidth;
	m_lNrOfFrames		= pXSprite->m_lNrOfFrames;
	
	m_pXSurfaceSprite	= pXSprite->m_pXSurfaceSprite;

	// Referenzzaehler erhoehen
	m_pXSurfaceSprite->addRef();

	// Initialisierung erfolgt
	m_bIsInitialized	= true;

	#if defined(XDEBUG) || defined(XSPRITEDEBUG)
	XLogging::log("XSprite::initialize > Sprite aus Vorlage erzeugt \"");
	XLogging::log(m_lpstrFilename);
	XLogging::log("\"\n");
	#endif

	return DD_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSprite::uninitialize
/////////////////////////////////////////////////////////////////////////////////////
// Info:			gibt das Sprite wieder frei
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSprite::uninitialize(void)
{
	if (m_bIsInitialized==false)
	{
		#if defined(XDEBUG) || defined(XSPRITEDEBUG)
		XLogging::log("XSprite::uninitialize > (WARNUNG!!!) Sprite nicht initialisiert\n");
		#endif

		return DDERR_INVALIDOBJECT;
	}
	
	if (m_pXSurfaceSprite)
	{
		m_pXSurfaceSprite->uninitialize();
		
		if (!m_pXSurfaceSprite->getRefCount())
		{
			delete m_pXSurfaceSprite;
			m_pXSurfaceSprite	= NULL;
		}
	}

	#if defined(XDEBUG) || defined(XSPRITEDEBUG)
	XLogging::log("XSprite::uninitialize > Sprite freigegeben \"");
	XLogging::log(m_lpstrFilename);
	XLogging::log("\"\n");
	#endif

	if (m_lpstrFilename)
	{
		delete [] m_lpstrFilename;
		m_lpstrFilename	= NULL;
	}

	// Member ruecksetzen
	resetData();

	return DD_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSprite::blit
/////////////////////////////////////////////////////////////////////////////////////
// Info:			kopiert den aktuellen Frame des Sprites auf die Surface
//					lpDestinationXSurface
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSprite::blit(XSurface*	lpDestinationXSurface,
					  DWORD		dwFlags)
{
	RECT	SrcRect;

	#if defined(XDEBUG) || defined(XSPRITEDEBUG)
	if (m_bIsInitialized==false)
	{
		XLogging::log("XSprite::blit > (FEHLER!!!) Sprite nicht initialisiert\n");
		
		return DDERR_INVALIDOBJECT;
	}
	if (!lpDestinationXSurface)
	{
		XLogging::log("XSprite::blit > (FEHLER!!!) Ziel-Surface existiert nicht\n");
		
		return DDERR_INVALIDOBJECT;
	}
	if ((m_lFrame<0) ||
		(m_lFrame>=m_lNrOfFrames))
	{
		XLogging::log("XSprite::blit > (FEHLER!!!) Frame ");
		XLogging::log(m_lFrame);
		XLogging::log(" nicht vorhanden \"");
		XLogging::log(m_lpstrFilename);
		XLogging::log("\"\n");
		
		return DDERR_INVALIDPARAMS;
	}
	#endif

	if (m_bVisible==false) 
	{
		return DD_OK;
	}

	// Quellrechteck
	SrcRect.left	= (m_lFrame%m_lFramesPerRow)*m_lFrameWidth;
	SrcRect.top		= (m_lFrame/m_lFramesPerRow)*m_lFrameHeight;
	SrcRect.right	= SrcRect.left+m_lFrameWidth;
	SrcRect.bottom	= SrcRect.top+m_lFrameHeight;

	return m_pXSurfaceSprite->blit(lpDestinationXSurface,m_lX,m_lY,&SrcRect,dwFlags);
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSprite::animate
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Animationsfunktion. Virtuell, damit sie in Klassen, die von
//					XSprite erben, ueberschrieben werden kann.
//					Der Rueckgabewert wird von XSpriteManager verwendet. Beim
//					Rueckgabewert "false", wird das Sprite aus den verwalteten
//					Sprites des Spritemanagers ausgeschlossen.
/////////////////////////////////////////////////////////////////////////////////////

bool XSprite::animate(void)
{
	return true;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSprite::restore
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Restauriert ein verlorenes Sprite. Laedt auch die Bitmap-Daten
//					neu.
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XSprite::restore(void)
{
	HRESULT				hDDResult;

	if (m_bIsInitialized==false)
	{
		#if defined(XDEBUG) || defined(XSPRITEDEBUG)
		XLogging::log("XSprite::restore > (FEHLER!!!) Sprite nicht initialisiert\n");
		#endif

		return DDERR_INVALIDOBJECT;
	}

	// Surface restoren,...
	hDDResult	= m_pXSurfaceSprite->restore();

	#if defined(XDEBUG) || defined(XSPRITEDEBUG)
	if (hDDResult!=DD_OK)
	{
		XLogging::log("XSprite::restore > (FEHLER!!!) restore fehlgeschlagen\n");
	}
	else
	{
		XLogging::log("XSprite::restore > Sprite restored \"");
		XLogging::log(m_lpstrFilename);
		XLogging::log("\"\n");
	}
	#endif

	return hDDResult;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSprite::getFrameHeight
/////////////////////////////////////////////////////////////////////////////////////
// Info:			gibt die Framehoehe zurueck
/////////////////////////////////////////////////////////////////////////////////////

long XSprite::getFrameHeight(void)
{
	return m_lFrameHeight;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XSprite::getFrameWidth
/////////////////////////////////////////////////////////////////////////////////////
// Info:			gibt die Framebreite zurueck
/////////////////////////////////////////////////////////////////////////////////////

long XSprite::getFrameWidth(void)
{
	return m_lFrameWidth;
}

/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	XSprite.cpp
/////////////////////////////////////////////////////////////////////////////////////