/////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		XSurface.h
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			pHreak
//
// Erstelldatum:	23.08.2000
// letzte Aenderung:15.03.2001
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Die Klasse XSurface verwaltet eine DirectDraw7-Surface.
//					Die Klasse stellt Funktionen zum Laden von Grafiken aus .BMP-
//					Dateien zur Verfuegung. Zusaetzliche Methoden erlauben blit-
//					Operationen in andere XSurfaces
/////////////////////////////////////////////////////////////////////////////////////
// Copyright 2000/2001 by x-treme
/////////////////////////////////////////////////////////////////////////////////////

#define DIRECTDRAW_VERSION 0x0700 // Dann klappts auch mit dem "DirectX>7-SDK"-Nachbarn

#define WIN32_LEAN_AND_MEAN		// Selten genutzte Teile aus Win-Header ausschliessen

#ifndef _XSURFACE_H
#define _XSURFACE_H

// includes
#include <windows.h>
#include <ddraw.h>
#include <d3d.h>

// XGame-includes
#define _XDISPLAY_H
#include "XDisplay.h"
#include "XLogging.h"

// Klassen-Deklaration

class	XSurface
{
public:
	XSurface(void);
	virtual ~XSurface(void);
	
	virtual HRESULT initialize(XDisplay*	pXDisplay,
							   LPSTR		lpstrFilename,
							   LPDDCOLORKEY	lpDDColorKey	= NULL,
							   DWORD		dwFlags			= DDSCAPS_OFFSCREENPLAIN);

	virtual HRESULT initialize(XDisplay*	pXDisplay,
							   long			lWidth,
							   long			lHeight,
							   LPDDCOLORKEY	lpDDColorKey	= NULL,
							   DWORD		dwFlags			= DDSCAPS_OFFSCREENPLAIN);

	virtual HRESULT uninitialize(void);

	virtual HRESULT restore(void);

	virtual void setClippingRect(LPRECT	lpClippingRect);
	virtual RECT getClippingRect(void);

	HRESULT blit(XSurface*	lpDestinationXSurface,
				 long		lX				= 0,
				 long		lY				= 0,
				 LPRECT		lpRectSource	= NULL,
				 DWORD		dwFlags			= DDBLT_WAIT);
	
	HRESULT blitAlpha(XDisplay*		lpXDisplay,
					  long			lX		= 0,
					  long			lY		= 0,
					  unsigned char	ucAlpha	= 255,
					  DWORD			dwFlags	= DDBLT_WAIT);

	HRESULT lock(LPVOID*	lplpSurface,
				 LPRECT		lpRectToLock = NULL,
				 DWORD		dwFlags = DDLOCK_WAIT);

	HRESULT unlock(void);

	HRESULT loadBitmapIntoSurface(LPSTR	lpstrFilename);

	static HRESULT getBitmapSize(LPSTR	lpstrFilename,
								 LPLONG	plWidth,
								 LPLONG	plHeight);

	long getWidth(void);
	long getHeight(void);

	long addRef(void);
	long getRefCount(void);

protected:
	// Nachfolgende Datenmember und Methoden sind "nur" protected,
	// damit die erbende Klasse XDisplay Zugriff auf sie hat.
	virtual void resetData(void);

	LPDIRECTDRAWSURFACE7	m_lpSurface;
	long					m_lWidth;
	long					m_lHeight;
	bool					m_bIsInitialized;
	RECT					m_rectClippingRect;

private:
	long					m_lRefCount;
	bool					m_bIsLocked;
	LPRECT					m_lpRectLocked;
	LPSTR					m_lpstrFilename;
	float					m_ftu;
	float					m_ftv;
};

#endif//_XSURFACE_H

class	XSurface;

/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	XSurface.h
/////////////////////////////////////////////////////////////////////////////////////